@@ -22,12 +22,6 @@
 #include "config.h"
 
 #include "pango-ot-private.h"
-#include "pango-impl-utils.h"
-#include FT_TRUETYPE_TABLES_H
-
-#if (!GLIB_CHECK_VERSION(2,29,15))
-#define G_UNICODE_SPACING_MARK G_UNICODE_COMBINING_MARK
-#endif
 
 static void pango_ot_info_finalize   (GObject *object);
 
@@ -101,12 +95,6 @@ pango_ot_info_get (FT_Face face)
   return info;
 }
 
-hb_face_t *
-_pango_ot_info_get_hb_face (PangoOTInfo *info)
-{
-  return info->hb_face;
-}
-
 static hb_tag_t
 get_hb_table_type (PangoOTTableType table_type)
 {
@@ -324,19 +312,3 @@ pango_ot_info_list_features  (PangoOTInfo      *info,
 
   return result;
 }
-
-void
-_pango_ot_info_substitute  (const PangoOTInfo    *info,
-			    const PangoOTRuleset *ruleset,
-			    PangoOTBuffer        *buffer)
-{
-  /* Deprecated. */
-}
-
-void
-_pango_ot_info_position    (const PangoOTInfo    *info,
-			    const PangoOTRuleset *ruleset,
-			    PangoOTBuffer        *buffer)
-{
-  /* Deprecated. */
-}
